package com.mortals.xhx.module.record.service.impl;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.core.util.StrUtil;
import com.deepoove.poi.XWPFTemplate;
import com.deepoove.poi.config.Configure;
import com.deepoove.poi.config.ConfigureBuilder;
import com.deepoove.poi.data.PictureRenderData;
import com.deepoove.poi.data.PictureType;
import com.deepoove.poi.data.Pictures;
import com.deepoove.poi.plugin.table.MultipleRowTableRenderPolicy;
import com.deepoove.poi.util.RegexUtils;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.mortals.framework.model.PageInfo;
import com.mortals.xhx.base.system.upload.service.UploadService;
import com.mortals.xhx.common.code.ClassifyType;
import com.mortals.xhx.common.code.GenerateStatus;
import com.mortals.xhx.common.utils.ExportDocUtil;
import com.mortals.xhx.common.utils.StringUtils;
import com.mortals.xhx.common.utils.WordUtil;
import com.mortals.xhx.module.certificate.model.CertificateCatalogEntity;
import com.mortals.xhx.module.certificate.model.CertificateClassifyEntity;
import com.mortals.xhx.module.certificate.service.CertificateCatalogService;
import com.mortals.xhx.module.certificate.service.CertificateClassifyService;
import com.mortals.xhx.module.record.model.ApplyLogQuery;
import com.mortals.xhx.module.record.model.DocTemplateVO;
import com.mortals.xhx.module.record.service.PrintWaitQueueService;
import org.apache.http.entity.ContentType;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.xhx.module.record.dao.ApplyLogDao;
import com.mortals.xhx.module.record.model.ApplyLogEntity;
import com.mortals.xhx.module.record.service.ApplyLogService;
import org.springframework.util.ObjectUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.ByteArrayOutputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.*;
import java.util.stream.Collectors;

/**
* ApplyLogService
* 证照申请 service实现
*
* @author zxfei
* @date 2022-10-14
*/
@Service("applyLogService")
public class ApplyLogServiceImpl extends AbstractCRUDServiceImpl<ApplyLogDao, ApplyLogEntity, Long> implements ApplyLogService {

    @Value("${upload.path}")
    private String filePath;

    @Autowired
    private UploadService uploadService;

    @Autowired
    private CertificateCatalogService certificateCatalogService;

    @Autowired
    private CertificateClassifyService certificateClassifyService;

    @Autowired
    private PrintWaitQueueService printWaitQueueService;

    @Override
    protected ApplyLogEntity findBefore(ApplyLogEntity entity, PageInfo pageInfo, Context context) throws AppException {
        ApplyLogQuery query = new ApplyLogQuery();
        BeanUtils.copyProperties(entity,query);
        if(StringUtils.isNotEmpty(entity.getQuery())){
            StringBuffer condition = new StringBuffer("%");
            condition.append(entity.getQuery()).append("%");
            ApplyLogQuery condition1 = new ApplyLogQuery();
            ApplyLogQuery condition2 = new ApplyLogQuery();
            condition1.setHolderName(condition.toString());
            condition2.setCertificateCode(condition.toString());
            List<ApplyLogQuery> orConditionList = new ArrayList<>();
            orConditionList.add(condition1);
            orConditionList.add(condition2);
            query.setOrConditionList(orConditionList);
        }
        return query;
    }

    @Override
    protected void validData(ApplyLogEntity entity, Context context) throws AppException {
        if(StringUtils.isEmpty(entity.getCertificateCode())){
            throw new AppException("证照编码不能为空");
        }
        if(entity.getIssueTime()==null){
            throw new AppException("颁发时间不能为空");
        }
        if(StringUtils.isEmpty(entity.getPickerName())){
            throw new AppException("取件人姓名不能为空");
        }
        if(entity.getHolderType()==null){
            throw new AppException("持有者类型不能为空");
        }
        if(entity.getHolderIdType()==null){
            throw new AppException("持有者证件类型不能为空");
        }
        if(StringUtils.isEmpty(entity.getHolderIDCardNo())){
            throw new AppException("持有者证件号码不能为空");
        }
        if(StringUtils.isEmpty(entity.getPrivateID())){
            throw new AppException("专网ID不能为空");
        }
        if(StringUtils.isEmpty(entity.getEnterpriseName())){
            throw new AppException("企业名称不能为空");
        }
        if(StringUtils.isEmpty(entity.getHolderName())){
            throw new AppException("持有者姓名不能为空");
        }
//        if(StringUtils.isEmpty(entity.getOriFormContent())){
//            throw new AppException("正本表单不能为空");
//        }
        if(entity.getCatalogId()==null){
            throw new AppException("目录ID不能为空");
        }
        super.validData(entity, context);
    }

    @Override
    protected void saveBefore(ApplyLogEntity entity, Context context) throws AppException {
        entity.setGenerateStatus(GenerateStatus.NOT.getValue());
    }

    @Override
    public String previewOriginal(ApplyLogEntity applyLogEntity, Context context) throws AppException {
        validData(applyLogEntity,context);
        CertificateCatalogEntity catalog = certificateCatalogService.get(applyLogEntity.getCatalogId());
        applyLogEntity.setCatalogCode(catalog.getCatalogCode());
        DocTemplateVO docTemplate = new DocTemplateVO(catalog.getOriginalUrl(),applyLogEntity.getOriFormContent());
        return preview(docTemplate, context);
    }

    @Override
    public String previewDuplicate(ApplyLogEntity applyLogEntity, Context context) throws AppException {
        validData(applyLogEntity,context);
        CertificateCatalogEntity catalog = certificateCatalogService.get(applyLogEntity.getCatalogId());
        applyLogEntity.setCatalogCode(catalog.getCatalogCode());
        DocTemplateVO docTemplate = new DocTemplateVO(catalog.getDuplicateUrl(),catalog.getDupFormContent());
        return preview(docTemplate, context);
    }

    @Override
    public void generateOriginal(ApplyLogEntity applyLogEntity, Context context) throws AppException {
        validData(applyLogEntity,context);
        CertificateCatalogEntity catalog = certificateCatalogService.get(applyLogEntity.getCatalogId());
        applyLogEntity.setCatalogCode(catalog.getCatalogCode());
        DocTemplateVO docTemplate = new DocTemplateVO(catalog.getOriginalUrl(),applyLogEntity.getOriFormContent());
        String paths = preview(docTemplate, context);
        String[] vals = paths.split(";");
        applyLogEntity.setOriginalUrl(vals[0]);
        applyLogEntity.setOriPreview(vals[1]);
        applyLogEntity.setCreateTime(new Date());
        if(context!=null && context.getUser()!=null) {
            applyLogEntity.setCreateUserId(context.getUser().getId());
        }
        applyLogEntity.setGenerateStatus(GenerateStatus.ORIGINAL.getValue());
        dao.insert(applyLogEntity);
        printWaitQueueService.creatWaitQueueByApply(applyLogEntity, GenerateStatus.ORIGINAL.getValue(),context);
    }

    @Override
    public void generateDuplicate(ApplyLogEntity applyLogEntity, Context context) {
        validData(applyLogEntity,context);
        CertificateCatalogEntity catalog = certificateCatalogService.get(applyLogEntity.getCatalogId());
        applyLogEntity.setCatalogCode(catalog.getCatalogCode());
        DocTemplateVO docTemplate = new DocTemplateVO(catalog.getDuplicateUrl(),applyLogEntity.getDupFormContent());
        String paths = preview(docTemplate, context);
        String[] vals = paths.split(";");
        applyLogEntity.setDuplicateUrl(vals[0]);
        applyLogEntity.setDupPreview(vals[1]);
        applyLogEntity.setCreateTime(new Date());
        if(context!=null && context.getUser()!=null) {
            applyLogEntity.setCreateUserId(context.getUser().getId());
        }
        applyLogEntity.setGenerateStatus(GenerateStatus.DUPLICATE.getValue());
        dao.insert(applyLogEntity);
        printWaitQueueService.creatWaitQueueByApply(applyLogEntity, GenerateStatus.DUPLICATE.getValue(),context);
    }

    @Override
    public void generateAll(ApplyLogEntity applyLogEntity, Context context) throws AppException {
        if(context==null || context.getUser()==null){
            throw new AppException("未登录或者登录过期");
        }
        validData(applyLogEntity,context);
        CertificateCatalogEntity catalog = certificateCatalogService.get(applyLogEntity.getCatalogId());
        applyLogEntity.setCatalogCode(catalog.getCatalogCode());
        DocTemplateVO docTemplate = new DocTemplateVO(catalog.getOriginalUrl(),applyLogEntity.getOriFormContent());
        String paths = preview(docTemplate, context);
        String[] vals = paths.split(";");
        applyLogEntity.setOriginalUrl(vals[0]);
        applyLogEntity.setOriPreview(vals[1]);
        CertificateClassifyEntity classifyEntity = certificateClassifyService.get(catalog.getClassifyId());
        if(classifyEntity!=null && classifyEntity.getClassifyType()== ClassifyType.PRINCIPAL_AND_VICE.getValue()) {
            docTemplate.setFormContent(applyLogEntity.getDupFormContent());
            docTemplate.setTemplatePath(catalog.getDuplicateUrl());
            String paths1 = preview(docTemplate, context);
            String[] vals1 = paths1.split(";");
            applyLogEntity.setDuplicateUrl(vals1[0]);
            applyLogEntity.setDupPreview(vals1[1]);
        }
        applyLogEntity.setCreateTime(new Date());
        if(context!=null && context.getUser()!=null) {
            applyLogEntity.setCreateUserId(context.getUser().getId());
        }
        applyLogEntity.setGenerateStatus(GenerateStatus.ALL.getValue());
        dao.insert(applyLogEntity);
        printWaitQueueService.creatWaitQueueByApply(applyLogEntity, GenerateStatus.ALL.getValue(),context);
    }

    private String preview(DocTemplateVO docTemplate,Context context){
        String rootPath = this.filePath.endsWith("/") ? this.filePath : this.filePath + "/";
        //转换表单参数为map集合
        ObjectMapper mapper = new ObjectMapper();
        ConfigureBuilder builder = Configure.builder();
        builder.buildGrammerRegex(RegexUtils.createGeneral("{{", "}}"));

        try {
            Map<String, Object> data = mapper.readValue(docTemplate.getFormContent(), Map.class);
            Map<String, Object> addMap = new HashMap<>();
            //遍历查看是否有图片数据，多选框
            data.entrySet().stream().forEach(entry -> {
                if (entry.getKey().indexOf("@image") != -1) {
                    //获取值 读取本地图片
                    String imagepath = uploadService.getFilePath(entry.getValue().toString());
                    log.info("Image path: " + imagepath);
                    try {
                        PictureRenderData pictureRenderData = Pictures.ofStream(new FileInputStream(imagepath), PictureType.JPEG)
                                .size(100, 120).create();
                        addMap.put(StrUtil.removePrefixIgnoreCase(entry.getKey(),"@"),pictureRenderData);
                        //entry.setValue(pictureRenderData);
                    } catch (FileNotFoundException e) {
                        log.error("error", e);
                    }
                }
                //构建填充复选框值
                if (entry.getKey().indexOf("ck_") != -1 && entry.getKey().indexOf("<") != -1) {
                    //获取复选框选项，并渲染值
                    String ckStr = com.mortals.framework.util.StringUtils.substringBetween(entry.getKey(), "<", ">");
                    //获取所有选项
                    List<String> checkboxs = StrUtil.splitTrim(ckStr, "-", -1);
                    //选中的项
                    List<String> ckList = Convert.toList(String.class, entry.getValue());
                    //合并
                    String ckStrs = checkboxs.stream().map(item -> {
                        for (String checked : ckList) {
                            if (checked.equals(item)) {
                                Map<Boolean, String> map = new HashMap<>();
                                map.put(true, item);
                                return map;
                            }
                        }
                        Map<Boolean, String> map = new HashMap<>();
                        map.put(false, item);
                        return map;
                    }).map(m -> m.entrySet().stream().map(m1 -> ExportDocUtil.createCheckBoxStrOther(m1.getKey(), m1.getValue())).findFirst().orElseGet(() -> "")
                    ).collect(Collectors.joining(""));
                    entry.setValue(ckStrs);
                }
                //TODO 单选
                //绑定动态表单
                if (entry.getKey().indexOf("dt_") != -1) {
                    builder.bind(entry.getKey(), new MultipleRowTableRenderPolicy());
                }
            });
            if(addMap.size()>0){
                data.putAll(addMap);
            }
            //查询是否有多选框，
            //获取模板文件地址
            String path = rootPath + docTemplate.getTemplatePath();
            String templateName = FileUtil.getName(path);
            Configure config = builder.build();
            XWPFTemplate template = null;
            ByteArrayOutputStream byteArrayOutputStream = null;
            try {
                template = XWPFTemplate.compile(path, config).render(data);
                byteArrayOutputStream = new ByteArrayOutputStream();
                template.write(byteArrayOutputStream);

                MultipartFile multipartFile = new MockMultipartFile(templateName, templateName,
                        ContentType.APPLICATION_OCTET_STREAM.toString(), byteArrayOutputStream.toByteArray());
                String mergedocPath = uploadService.saveFileUpload(multipartFile, "/mergedoc", context.getUser());
                log.info("mergedocPath:" + mergedocPath);
                String mergedoc = rootPath + mergedocPath;
                //转换预览图片
                String fileName = RandomUtil.randomNumbers(12) + ".jpg";
                String preView = this.filePath + "/preview/" + fileName;
                WordUtil.convertWordToJPEG(mergedoc, preView);
                log.info("preView:" + "/preview/" + fileName);
                //下载地址拼装
                String returnStr = mergedocPath + ";" + "/preview/" + fileName;
                return returnStr;
            } catch (Exception e) {
                log.error("渲染模板失败：", e);
            } finally {
                try {
                    template.close();
                    byteArrayOutputStream.close();
                } catch (IOException e) {
                    log.error("ioEx：", e);
                }
            }
        } catch (JsonProcessingException e) {
            log.error("转换异常", e);
            throw new AppException("表单参数转换异常:" + e.getMessage());
        }
        return "";
    }
}