package com.mortals.xhx.module.record.service.impl;
import com.alibaba.fastjson.JSONObject;
import com.mortals.framework.util.HttpUtil;
import com.mortals.xhx.common.code.*;
import com.mortals.xhx.module.certificate.model.CertificateCatalogEntity;
import com.mortals.xhx.module.certificate.model.CertificateClassifyEntity;
import com.mortals.xhx.module.certificate.model.CertificateClassifyQuery;
import com.mortals.xhx.module.certificate.service.CertificateCatalogService;
import com.mortals.xhx.module.certificate.service.CertificateClassifyService;
import com.mortals.xhx.module.record.dao.PrintLogDao;
import com.mortals.xhx.module.record.dao.RetainLogDao;
import com.mortals.xhx.module.record.model.ApplyLogEntity;
import com.mortals.xhx.module.record.model.PrintLogEntity;
import com.mortals.xhx.module.record.model.RetainLogEntity;
import com.mortals.xhx.module.record.service.PrintLogService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.xhx.module.record.dao.PrintWaitQueueDao;
import com.mortals.xhx.module.record.model.PrintWaitQueueEntity;
import com.mortals.xhx.module.record.service.PrintWaitQueueService;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
* PrintWaitQueueService
* 证照打印记录 service实现
*
* @author zxfei
* @date 2022-10-22
*/
@Service("printWaitQueueService")
public class PrintWaitQueueServiceImpl extends AbstractCRUDServiceImpl<PrintWaitQueueDao, PrintWaitQueueEntity, Long> implements PrintWaitQueueService {

    /** 系统ip端口 */
    @Value("${system.ip:http://192.168.0.98:11075/}")
    private String ip;
    /** 打印api接口地址 */
    @Value("${print.aip:http://192.168.0.58:8037/api/print/GetHardWareInfo?GetType=78}")
    private String printApi;
    /** 打印机设备ip */
    @Value("${system.print:192.168.0.11}")
    private String printIp;

    @Autowired
    private PrintLogDao printLogDao;
    @Autowired
    private RetainLogDao retainLogDao;
    @Autowired
    private CertificateCatalogService certificateCatalogService;
    @Autowired
    private CertificateClassifyService certificateClassifyService;

    @Override
    protected void findAfter(PrintWaitQueueEntity entity, Context context, List<PrintWaitQueueEntity> list) throws AppException {
        for(PrintWaitQueueEntity printWaitQueueEntity:list){
            CertificateCatalogEntity catalogEntity = certificateCatalogService.get(printWaitQueueEntity.getCatalogId());
            printWaitQueueEntity.setPaperSource(catalogEntity.getPaperSource());
            printWaitQueueEntity.setPrinterIp(catalogEntity.getPrinterIp());
            printWaitQueueEntity.setPrinterName(catalogEntity.getPrinterName());
        }
    }

    @Override
    public void creatWaitQueueByApply(ApplyLogEntity applyLogEntity, int generateType, Context context) throws AppException {
        //CertificateCatalogEntity catalogEntity = certificateCatalogService.get(applyLogEntity.getCatalogId());
        if(generateType== GenerateStatus.ALL.getValue()){
            PrintWaitQueueEntity original = creatWaitQueue(applyLogEntity,GenerateStatus.ORIGINAL.getValue());
            dao.insert(original);
            PrintWaitQueueEntity duplicate = creatWaitQueue(applyLogEntity,GenerateStatus.DUPLICATE.getValue());
            dao.insert(duplicate);
        }else {
            PrintWaitQueueEntity printWaitQueueEntity = creatWaitQueue(applyLogEntity,generateType);
            dao.insert(printWaitQueueEntity);
        }
    }

    private PrintWaitQueueEntity creatWaitQueue(ApplyLogEntity applyLogEntity,int generateType){
        PrintWaitQueueEntity waitQueueEntity = new PrintWaitQueueEntity();
        waitQueueEntity.setApplyId(applyLogEntity.getId());
        waitQueueEntity.setCatalogId(applyLogEntity.getCatalogId());
        waitQueueEntity.setCatalogCode(applyLogEntity.getCatalogCode());
        waitQueueEntity.setCertificateName(applyLogEntity.getCertificateName());
        waitQueueEntity.setCertificateCode(applyLogEntity.getCertificateCode());
        waitQueueEntity.setEnterpriseName(applyLogEntity.getEnterpriseName());
        waitQueueEntity.setHolderName(applyLogEntity.getHolderName());
        waitQueueEntity.setHolderIDCardNo(applyLogEntity.getHolderIDCardNo());
        waitQueueEntity.setPickerName(applyLogEntity.getPickerName());
        waitQueueEntity.setPickerIDCardNo(applyLogEntity.getPickerIDCardNo());
        if(generateType== GenerateStatus.DUPLICATE.getValue()) {
            waitQueueEntity.setCertificateType(CertificateType.DUPLICATE.getValue());
            waitQueueEntity.setPreviewUrl(applyLogEntity.getDupPreview());
            waitQueueEntity.setCertificateUrl(applyLogEntity.getDuplicateUrl());
        }else{
            waitQueueEntity.setCertificateType(CertificateType.ORIGINAL.getValue());
            waitQueueEntity.setPreviewUrl(applyLogEntity.getOriPreview());
            waitQueueEntity.setCertificateUrl(applyLogEntity.getOriginalUrl());
        }
        waitQueueEntity.setPrintStatus(YesNoEnum.NO.getValue());
        waitQueueEntity.setCreateUserId(applyLogEntity.getCreateUserId());
        waitQueueEntity.setCreateTime(new Date());
        return waitQueueEntity;
    }

    @Override
    public void doPrint(Long id,int printStatus,String statusRemark) {
        PrintWaitQueueEntity waitQueueEntity = this.dao.get(id);
        if(waitQueueEntity==null){
            throw new AppException("数据不存在或已修改");
        }
        PrintWaitQueueEntity data = new PrintWaitQueueEntity();
        data.setId(id);
        data.setPrintStatus(YesNoEnum.YES.getValue());
        if(printStatus==PrintStatus.SUCCESS.getValue()){
            doPrintSuccess(waitQueueEntity);
            data.setPrintStatus(YesNoEnum.YES.getValue());
        }else {
            doPrintFail(waitQueueEntity,statusRemark);
            data.setPrintStatus(-1);
        }
        data.setUpdateTime(new Date());
        dao.update(data);

    }

    private void doPrintFail(PrintWaitQueueEntity waitQueueEntity,String statusRemark){
        Date now = new Date();
        PrintLogEntity printLogEntity = new PrintLogEntity();
        printLogEntity.setCatalogId(waitQueueEntity.getCatalogId());
        printLogEntity.setCatalogCode(waitQueueEntity.getCatalogCode());
        if(waitQueueEntity.getCertificateType()==CertificateType.ORIGINAL.getValue()){
            printLogEntity.setOriginalCount(1);
            printLogEntity.setDuplicateCount(0);
        }else {
            printLogEntity.setOriginalCount(0);
            printLogEntity.setDuplicateCount(1);
        }
        printLogEntity.setCertificateName(waitQueueEntity.getCertificateName());
        printLogEntity.setCertificateCode(waitQueueEntity.getCertificateCode());
        printLogEntity.setEnterpriseName(waitQueueEntity.getEnterpriseName());
        printLogEntity.setHolderName(waitQueueEntity.getHolderName());
        printLogEntity.setHolderIDCardNo(waitQueueEntity.getHolderIDCardNo());
        printLogEntity.setPickerName(waitQueueEntity.getPickerName());
        printLogEntity.setPickerIDCardNo(waitQueueEntity.getPickerIDCardNo());

        printLogEntity.setPrintDate(now);
        printLogEntity.setCreateUserId(waitQueueEntity.getCreateUserId());
        printLogEntity.setCreateTime(now);
        printLogEntity.setPrintStatus(PrintStatus.FAIL.getValue());
        printLogEntity.setStatusRemark(statusRemark);
        printLogDao.insert(printLogEntity);
    }

    private void doPrintSuccess(PrintWaitQueueEntity waitQueueEntity){
        RetainLogEntity retainLogEntity = new RetainLogEntity();
        Date now = new Date();
        retainLogEntity.setCatalogId(waitQueueEntity.getCatalogId());
        retainLogEntity.setCatalogCode(waitQueueEntity.getCatalogCode());
        retainLogEntity.setCertificateName(waitQueueEntity.getCertificateName());
        retainLogEntity.setCertificateCode(waitQueueEntity.getCertificateCode());
        retainLogEntity.setEnterpriseName(waitQueueEntity.getEnterpriseName());
        retainLogEntity.setHolderName(waitQueueEntity.getHolderName());
        retainLogEntity.setHolderIDCardNo(waitQueueEntity.getHolderIDCardNo());
        retainLogEntity.setCertificateStatus(CertificateStatus.NORMAL.getValue());
        retainLogEntity.setCreateUserId(waitQueueEntity.getCreateUserId());
        retainLogEntity.setCreateTime(now);

        PrintLogEntity printLogEntity = new PrintLogEntity();
        printLogEntity.setCatalogId(waitQueueEntity.getCatalogId());
        printLogEntity.setCatalogCode(waitQueueEntity.getCatalogCode());
        if(waitQueueEntity.getCertificateType()==CertificateType.ORIGINAL.getValue()){
            printLogEntity.setOriginalCount(1);
            printLogEntity.setDuplicateCount(0);

            retainLogEntity.setOriginalCount(1);
            retainLogEntity.setDuplicateCount(0);
            retainLogEntity.setOriginalUrl(waitQueueEntity.getCertificateUrl());
        }else {
            printLogEntity.setOriginalCount(0);
            printLogEntity.setDuplicateCount(1);

            retainLogEntity.setOriginalCount(0);
            retainLogEntity.setDuplicateCount(1);
            retainLogEntity.setDuplicateUrl(waitQueueEntity.getCertificateUrl());
        }
        printLogEntity.setCertificateName(waitQueueEntity.getCertificateName());
        printLogEntity.setCertificateCode(waitQueueEntity.getCertificateCode());
        printLogEntity.setEnterpriseName(waitQueueEntity.getEnterpriseName());
        printLogEntity.setHolderName(waitQueueEntity.getHolderName());
        printLogEntity.setHolderIDCardNo(waitQueueEntity.getHolderIDCardNo());
        printLogEntity.setPickerName(waitQueueEntity.getPickerName());
        printLogEntity.setPickerIDCardNo(waitQueueEntity.getPickerIDCardNo());

        printLogEntity.setPrintDate(now);
        printLogEntity.setCreateUserId(waitQueueEntity.getCreateUserId());
        printLogEntity.setCreateTime(now);
        printLogEntity.setPrintStatus(PrintStatus.SUCCESS.getValue());
        retainLogDao.insert(retainLogEntity);
        printLogDao.insert(printLogEntity);
        CertificateCatalogEntity catalogEntity = certificateCatalogService.get(waitQueueEntity.getCatalogId());
        CertificateClassifyQuery query = new CertificateClassifyQuery();
        query.setId(catalogEntity.getClassifyId());
        query.setTotalIncrement(1);
        query.setUpdateTime(now);
        certificateClassifyService.update(query);


    }
}